//	ArrestExe - File Audit Utility
//	Copyright (C) 2002-2006 AnonyMouse
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Project Includes:
#include "Stdafx.h"
#include "ArrestExe.h"
#include "ArrestExeDlg.h"

// General includes:
#include <shlwapi.h>

// Utility includes:
#include "FileUtils.h"
#include "StringUtils.h"
#include "MessageUtils.h"
#include "SystemUtils.h"
#include "DirDialog.h"

// Debug Codes:
#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CArrestExeDlg::CArrestExeDlg( CWnd* pParent ):CDialog( CArrestExeDlg::IDD, pParent )
{
	m_hIcon = AfxGetApp()->LoadIcon( IDR_MAINFRAME );
	m_hIcon16 = AfxGetApp()->LoadIcon( IDI_MAINFRAME_MINI );

	m_TextLogWriter = NULL;
	m_AuditThreadManager = NULL;
	m_DeleteServiceThreadManager = NULL;

	m_RDCWUseable = false;
}

CArrestExeDlg::~CArrestExeDlg()
{
	// Remove TaskTray Icon Here ( Removing In DoExit() Will Fail )
	CAppUtils::SetTaskTrayIcon( CAppUtils::TASKTRAY_NONE );

	// Delete Thread Manager
	if( m_AuditThreadManager != NULL )
	{
		delete m_AuditThreadManager;
	}
	if( m_DeleteServiceThreadManager != NULL )
	{
		delete m_DeleteServiceThreadManager;
	}

	// Delete TextLogWriter
	if( m_TextLogWriter != NULL )
	{
		delete m_TextLogWriter;
	}
}

void CArrestExeDlg::DoDataExchange( CDataExchange* pDX )
{
	CDialog::DoDataExchange( pDX );
	DDX_Control( pDX, IDC_EDIT_LOG, m_EditLog );
	DDX_Control( pDX, IDC_STAT_AUDIT1, m_StatAudit1 );
	DDX_Control( pDX, IDC_STAT_AUDIT2, m_StatAudit2 );
	DDX_Control( pDX, IDC_STAT_AUDIT3, m_StatAudit3 );
	DDX_Control( pDX, IDC_STAT_AUDIT4, m_StatAudit4 );
	DDX_Control( pDX, IDC_STAT_AUDIT5, m_StatAudit5 );
	DDX_Control( pDX, IDC_STAT_AUDIT_DIR1, m_StatAuditDir1 );
	DDX_Control( pDX, IDC_STAT_AUDIT_DIR2, m_StatAuditDir2 );
	DDX_Control( pDX, IDC_STAT_AUDIT_DIR3, m_StatAuditDir3 );
	DDX_Control( pDX, IDC_STAT_AUDIT_DIR4, m_StatAuditDir4 );
	DDX_Control( pDX, IDC_STAT_AUDIT_DIR5, m_StatAuditDir5 );
	DDX_Control( pDX, IDC_BTN_REF1, m_BtnRef1 );
	DDX_Control( pDX, IDC_BTN_REF2, m_BtnRef2 );
	DDX_Control( pDX, IDC_BTN_REF3, m_BtnRef3 );
	DDX_Control( pDX, IDC_BTN_REF4, m_BtnRef4 );
	DDX_Control( pDX, IDC_BTN_REF5, m_BtnRef5 );
	DDX_Control( pDX, IDC_BTN_CLEAR1, m_BtnClear1 );
	DDX_Control( pDX, IDC_BTN_CLEAR2, m_BtnClear2 );
	DDX_Control( pDX, IDC_BTN_CLEAR3, m_BtnClear3 );
	DDX_Control( pDX, IDC_BTN_CLEAR4, m_BtnClear4 );
	DDX_Control( pDX, IDC_BTN_CLEAR5, m_BtnClear5 );
	DDX_Control( pDX, IDC_STAT_EXTENSION, m_StatExtension );
	DDX_Control( pDX, IDC_EDIT_EXTENSION, m_EditExtension );
	DDX_Control( pDX, IDC_CHK_AUTORUN, m_ChkAutorun );
	DDX_Control( pDX, IDC_CHK_COMPLETE_DELETE, m_ChkCompleteDelete );
	DDX_Control( pDX, IDC_CHK_RUN_AS_TASKTRAY, m_ChkRunAsTaskTray );
	DDX_Control( pDX, IDC_CHK_USE_SCAN, m_ChkUseScan );
	DDX_Control( pDX, IDC_CHK_USE_START_SEARCH, m_ChkUseStartSearch );
	DDX_Control( pDX, IDC_CHK_TRAY_ON_START, m_ChkTrayOnStart );
	DDX_Control( pDX, IDC_BTN_EXECUTE, m_BtnExecute );
	DDX_Control( pDX, IDC_BTN_LOGCLEAR, m_BtnLogClear );
	DDX_Control( pDX, IDCANCEL, m_BtnExit );
	DDX_Control( pDX, IDC_STAT_GROUP, m_StatGroup );
	DDX_Control( pDX, IDC_STAT_OPTION, m_StatOption );
	DDX_Control( pDX, IDC_BTN_TASKTRAY, m_BtnTaskTray );
}

BEGIN_MESSAGE_MAP( CArrestExeDlg, CDialog )
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_WINDOWPOSCHANGING()
	ON_BN_CLICKED( IDC_BTN_REF1, OnBnClickedBtnRef1 )
	ON_BN_CLICKED( IDC_BTN_REF2, OnBnClickedBtnRef2 )
	ON_BN_CLICKED( IDC_BTN_REF3, OnBnClickedBtnRef3 )
	ON_BN_CLICKED( IDC_BTN_REF4, OnBnClickedBtnRef4 )
	ON_BN_CLICKED( IDC_BTN_REF5, OnBnClickedBtnRef5 )
	ON_BN_CLICKED( IDC_BTN_CLEAR1, OnBnClickedBtnClear1 )
	ON_BN_CLICKED( IDC_BTN_CLEAR2, OnBnClickedBtnClear2 )
	ON_BN_CLICKED( IDC_BTN_CLEAR3, OnBnClickedBtnClear3 )
	ON_BN_CLICKED( IDC_BTN_CLEAR4, OnBnClickedBtnClear4 )
	ON_BN_CLICKED( IDC_BTN_CLEAR5, OnBnClickedBtnClear5 )
	ON_BN_CLICKED( IDC_CHK_AUTORUN, OnBnClickedChkAutorun )
	ON_BN_CLICKED( IDC_CHK_COMPLETE_DELETE, OnBnClickedChkCompleteDelete )
	ON_BN_CLICKED( IDCANCEL, OnBnClickedCancel )
	ON_BN_CLICKED( IDC_BTN_EXECUTE, OnBnClickedBtnExecute )
	ON_BN_CLICKED( IDC_CHK_RUN_AS_TASKTRAY, OnBnClickedChkRunAsTasktray )
	ON_BN_CLICKED(IDC_BTN_LOGCLEAR, OnBnClickedBtnLogclear)
	ON_BN_CLICKED( IDC_BTN_TASKTRAY, OnBnClickedBtnTasktray )
	ON_BN_CLICKED( IDC_CHK_USE_SCAN, OnBnClickedChkUseScan )
	ON_BN_CLICKED( IDC_CHK_USE_START_SEARCH, OnBnClickedChkUseStartSearch )
	ON_BN_CLICKED( IDC_CHK_TRAY_ON_START, OnBnClickedChkTrayOnStart )
	ON_EN_KILLFOCUS( IDC_EDIT_EXTENSION, OnEnKillfocusEditExtension )
	ON_MESSAGE( WM_TASKTRAYMSG, OnTaskTrayMessage )
END_MESSAGE_MAP()

void CArrestExeDlg::OnSysCommand( UINT nID, LPARAM lParam )
{
	CDialog::OnSysCommand( nID, lParam );
}

void CArrestExeDlg::OnPaint() 
{
	if( IsIconic() )
	{
		CPaintDC dc( this );

		SendMessage( WM_ICONERASEBKGND, reinterpret_cast<WPARAM>( dc.GetSafeHdc() ), 0 );

		int cxIcon = GetSystemMetrics( SM_CXICON );
		int cyIcon = GetSystemMetrics( SM_CYICON );
		CRect rect;
		GetClientRect( &rect );
		int x = ( rect.Width() - cxIcon + 1 ) / 2;
		int y = ( rect.Height() - cyIcon + 1 ) / 2;

		dc.DrawIcon( x, y, m_hIcon );
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR CArrestExeDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>( m_hIcon );
}

BOOL CArrestExeDlg::PreTranslateMessage( MSG* pMsg )
{
	if( pMsg->message == WM_KEYDOWN )
	{
		if( pMsg->wParam == VK_ESCAPE )
		{
			return TRUE;
		}
	}
	else if( pMsg->message == WM_SYSCOMMAND )
	{
		switch( pMsg->wParam )
		{
		case ARRESTEXEDLG_SYSMENU_ADD_STARTUP:
			OnMenuSysStartUp( true );
			break;
		case ARRESTEXEDLG_SYSMENU_DEL_STARTUP:
			OnMenuSysStartUp( false );
			break;
		case ARRESTEXEDLG_SYSMENU_SHOW_HP:
			OnMenuSysAddShowHP();
			break;
		default:
			break;
		}
	}

	return CDialog::PreTranslateMessage(pMsg);
}

BOOL CArrestExeDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set LogText INFINITE
	m_EditLog.SetLimitText( 0 );

	// Init TrayIcon
	InitTrayIcon();

	// Make System Menu
	CMenu *lSysMenu = GetSystemMenu( NULL );
	lSysMenu->InsertMenu( 0, MF_BYPOSITION, ARRESTEXEDLG_SYSMENU_ADD_STARTUP, CStringUtils::GetResStr( IDS_MENU_SYS_ADD_STARTUP ) );
	lSysMenu->InsertMenu( 1, MF_BYPOSITION, ARRESTEXEDLG_SYSMENU_DEL_STARTUP, CStringUtils::GetResStr( IDS_MENU_SYS_DEL_STARTUP ) );
	lSysMenu->InsertMenu( 2, MF_BYPOSITION + MF_SEPARATOR );
	lSysMenu->InsertMenu( 3, MF_BYPOSITION, ARRESTEXEDLG_SYSMENU_SHOW_HP, CStringUtils::GetResStr( IDS_MENU_SYS_SHOW_HP ) );
	lSysMenu->InsertMenu( 4, MF_BYPOSITION + MF_SEPARATOR );
	UpdateStartUpMenuEnable();

	// Create LogWriter
	m_TextLogWriter = new CTextLogWriter( &m_EditLog );

	// Set Icons
	SetIcon(m_hIcon, TRUE);
	SetIcon(m_hIcon16, FALSE);

	// Check RDCW Useable
	m_RDCWUseable = CSystemUtils::IsReadDirectoryChangesWUseable();
	if( !m_RDCWUseable )
	{
		m_AuditOption.SetUseScan( true );
		m_ChkUseScan.EnableWindow( false );
	}

	// Load Audit Option
	LoadOption();

	// Run As TaskTray
	CAppUtils::SetMainWndVisible( !m_AuditOption.GetRunAsTaskTray(), true );

	// Auto Run
	if( m_AuditOption.GetAutoRun() )
	{
		DoSwitch();
	}

	return TRUE;
}

void CArrestExeDlg::InitTrayIcon()
{
	NOTIFYICONDATA* lNotifyIconData = CAppUtils::GetTaskTrayData();
	lNotifyIconData->cbSize = sizeof( NOTIFYICONDATA );
	lNotifyIconData->hWnd = m_hWnd;
	lNotifyIconData->uID = 0;
	lNotifyIconData->uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	lNotifyIconData->uCallbackMessage = WM_TASKTRAYMSG;
	_tcscpy( lNotifyIconData->szTip, _T( "ArrestExe" ) );
	CAppUtils::SetTaskTrayIcon( CAppUtils::TASKTRAY_STOP );
}

void CArrestExeDlg::LoadOption()
{
	// Load Audit Folder
	CStringArray* lAuditArray = m_AuditOption.GetAuditArray();
	m_StatAuditDir1.SetWindowText( lAuditArray->GetAt( 0 ) );
	m_StatAuditDir2.SetWindowText( lAuditArray->GetAt( 1 ) );
	m_StatAuditDir3.SetWindowText( lAuditArray->GetAt( 2 ) );
	m_StatAuditDir4.SetWindowText( lAuditArray->GetAt( 3 ) );
	m_StatAuditDir5.SetWindowText( lAuditArray->GetAt( 4 ) );

	// Load Extension List
	m_EditExtension.SetWindowText( m_AuditOption.GetExtensionList() );

	// Load AutoRun
	m_ChkAutorun.SetCheck( m_AuditOption.GetAutoRun() ? BST_CHECKED : BST_UNCHECKED );

	// Load RunAsTaskTray
	m_ChkRunAsTaskTray.SetCheck( m_AuditOption.GetRunAsTaskTray() ? BST_CHECKED : BST_UNCHECKED );

	// Load CompleteDelete
	m_ChkCompleteDelete.SetCheck( !m_AuditOption.GetCompleteDelete() ? BST_CHECKED : BST_UNCHECKED );

	// Load UseScan
	m_ChkUseScan.SetCheck( m_AuditOption.GetUseScan() ? BST_CHECKED : BST_UNCHECKED );

	// Load UseStartSearch
	m_ChkUseStartSearch.SetCheck( m_AuditOption.GetUseStartSearch() ? BST_CHECKED : BST_UNCHECKED );

	// Load TrayOnStart
	m_ChkTrayOnStart.SetCheck( m_AuditOption.GetTrayOnStart() ? BST_CHECKED : BST_UNCHECKED );
}

void CArrestExeDlg::SaveOption()
{
	// Save Audit Folder
	CStringArray* lAuditArray = m_AuditOption.GetAuditArray();
	lAuditArray->RemoveAll();
	CString lAuditDir = _T( "" );
	m_StatAuditDir1.GetWindowText( lAuditDir );
	lAuditArray->Add( lAuditDir );
	m_StatAuditDir2.GetWindowText( lAuditDir );
	lAuditArray->Add( lAuditDir );
	m_StatAuditDir3.GetWindowText( lAuditDir );
	lAuditArray->Add( lAuditDir );
	m_StatAuditDir4.GetWindowText( lAuditDir );
	lAuditArray->Add( lAuditDir );
	m_StatAuditDir5.GetWindowText( lAuditDir );
	lAuditArray->Add( lAuditDir );

	// Load Extension List
	CString lExtensionList = _T( "" );
	m_EditExtension.GetWindowText( lExtensionList );
	m_AuditOption.SetExtensionList( lExtensionList );

	// Load AutoRun
	m_AuditOption.SetAutoRun( m_ChkAutorun.GetCheck() == BST_CHECKED );

	// Load RunAsTaskTray
	m_AuditOption.SetRunAsTaskTray( m_ChkRunAsTaskTray.GetCheck() == BST_CHECKED );

	// Load CompleteDelete
	m_AuditOption.SetCompleteDelete( m_ChkCompleteDelete.GetCheck() != BST_CHECKED );

	// Load UseScan
	m_AuditOption.SetUseScan( m_ChkUseScan.GetCheck() == BST_CHECKED );

	// Load UseStartSearch
	m_AuditOption.SetUseStartSearch( m_ChkUseStartSearch.GetCheck() == BST_CHECKED );

	// Load TrayOnStart
	m_AuditOption.SetTrayOnStart( m_ChkTrayOnStart.GetCheck() == BST_CHECKED );

	// Do Save
	m_AuditOption.Save();
}

void CArrestExeDlg::ClearAuditDir( CStatic* ipAuditDirLabel )
{
	// Clear Path
	ipAuditDirLabel->SetWindowText( _T( "" ) );

	// Save Option
	SaveOption();
}

void CArrestExeDlg::SelectAuditDir( CStatic* ipAuditDirLabel )
{
	CDirDialog lDirDialog;

	// Show DirDialog
	lDirDialog.m_hParent = m_hWnd;
	lDirDialog.m_strTitle = CStringUtils::GetResStr( IDS_RES_DIRDIALOG_TITLE );
	BeginWaitCursor();
	if(!lDirDialog.DoBrowse()){
		EndWaitCursor();
		return;
	}
	EndWaitCursor();

	// Check Path
	CString lDirectory = lDirDialog.m_strPath;
	if( CFileUtils::IsSpecialFolder( lDirectory ) )
	{
		CMessageUtils::WMSG( CStringUtils::GetResStr( IDS_WRN_DANGER_PATH ) );
		return;
	}

	// Check Overlap
	if( IsAuditDirOverlap( ipAuditDirLabel, &m_StatAuditDir1, lDirectory ) ||
		IsAuditDirOverlap( ipAuditDirLabel, &m_StatAuditDir2, lDirectory ) ||
		IsAuditDirOverlap( ipAuditDirLabel, &m_StatAuditDir3, lDirectory ) ||
		IsAuditDirOverlap( ipAuditDirLabel, &m_StatAuditDir4, lDirectory ) ||
		IsAuditDirOverlap( ipAuditDirLabel, &m_StatAuditDir5, lDirectory ) )
	{
		CMessageUtils::WMSG( CStringUtils::GetResStr( IDS_WRN_OVERLAP_PATH ) );
		return;
	}

	// Set Path To Label
	ipAuditDirLabel->SetWindowText( lDirectory );

	// Save Option
	SaveOption();
}

bool CArrestExeDlg::IsAuditDirOverlap( CStatic* ipStatSelf, CStatic* ipStatTarget, CString ipDirPath )
{
	// Check Self
	if( ipStatSelf == ipStatTarget )
	{
		return false;
	}

	// Get Target Text
	CString lTargetPath = _T( "" );
	ipStatTarget->GetWindowText( lTargetPath );
	if( lTargetPath == _T( "" ) )
	{
		return false;
	}

	// Compare Overlap
	return( CFileUtils::IsPathOverlap( ipDirPath, lTargetPath ) );
}

void CArrestExeDlg::ValidateExtensionList()
{
	CString lExtensionList = _T( "" );
	m_EditExtension.GetWindowText( lExtensionList );
	m_EditExtension.SetWindowText( CAuditOption::GetValidExtensionList( lExtensionList ) );
	SaveOption();
}

void CArrestExeDlg::OnBnClickedBtnRef1()
{
	SelectAuditDir( &m_StatAuditDir1 );
}

void CArrestExeDlg::OnBnClickedBtnRef2()
{
	SelectAuditDir( &m_StatAuditDir2 );
}

void CArrestExeDlg::OnBnClickedBtnRef3()
{
	SelectAuditDir( &m_StatAuditDir3 );
}

void CArrestExeDlg::OnBnClickedBtnRef4()
{
	SelectAuditDir( &m_StatAuditDir4 );
}

void CArrestExeDlg::OnBnClickedBtnRef5()
{
	SelectAuditDir( &m_StatAuditDir5 );
}

void CArrestExeDlg::OnBnClickedBtnClear1()
{
	ClearAuditDir( &m_StatAuditDir1 );
}

void CArrestExeDlg::OnBnClickedBtnClear2()
{
	ClearAuditDir( &m_StatAuditDir2 );
}

void CArrestExeDlg::OnBnClickedBtnClear3()
{
	ClearAuditDir( &m_StatAuditDir3 );
}

void CArrestExeDlg::OnBnClickedBtnClear4()
{
	ClearAuditDir( &m_StatAuditDir4 );
}

void CArrestExeDlg::OnBnClickedBtnClear5()
{
	ClearAuditDir( &m_StatAuditDir5 );
}

void CArrestExeDlg::OnBnClickedChkAutorun()
{
	SaveOption();
}

void CArrestExeDlg::OnBnClickedChkCompleteDelete()
{
	SaveOption();
}

void CArrestExeDlg::OnBnClickedChkRunAsTasktray()
{
	SaveOption();
}

void CArrestExeDlg::OnBnClickedChkUseScan()
{
	SaveOption();
}

void CArrestExeDlg::OnBnClickedChkUseStartSearch()
{
	SaveOption();
}

void CArrestExeDlg::OnBnClickedChkTrayOnStart()
{
	SaveOption();
}

void CArrestExeDlg::OnBnClickedBtnLogclear()
{
	ClearWarning();
	m_EditLog.SetWindowText( _T( "" ) );
}

void CArrestExeDlg::OnBnClickedCancel()
{
	DoExit();
}

void CArrestExeDlg::OnEnKillfocusEditExtension()
{
	ValidateExtensionList();
}

void CArrestExeDlg::OnBnClickedBtnExecute()
{
	DoSwitch( m_AuditOption.GetTrayOnStart() );
}

void CArrestExeDlg::DoSwitch( bool ipTrayOnStart )
{
	if( !CAppUtils::IsRunning() )
	{
		ValidateExtensionList();

		if( ipTrayOnStart )
		{
			if( CAppUtils::GetMainWndVisible() )
			{
				CAppUtils::SetMainWndVisible( false );
			}
		}

		DoAudit();
	}
	else
	{
		DoCancel();
	}
}

void CArrestExeDlg::DoAudit()
{
	// Check Option
	if( !CheckOption() )
	{
		return;
	}

	BeginWaitCursor();

	// Create Extension Array
	CString lExtensionList = m_AuditOption.GetExtensionList();
	CStringArray lExtensionArray;
	int lPosition = 0;
	CString lExtension = lExtensionList.Tokenize( AUDITOPTION_EXTENSION_SEPARATER, lPosition );
	while( lExtension != _T( "" ) )
	{
		lExtensionArray.Add( lExtension );
		lExtension = lExtensionList.Tokenize( AUDITOPTION_EXTENSION_SEPARATER, lPosition );
	}

	// Create Delete Service Thread Manager
	if( m_DeleteServiceThreadManager != NULL )
	{
		delete m_DeleteServiceThreadManager;
	}
	m_DeleteServiceThreadManager = new CThreadManager( m_TextLogWriter );

	// Create Delete Service Thread
	CDeleteServiceThread* lDeleteServiceThread = new CDeleteServiceThread( m_AuditOption.GetCompleteDelete(), ARRESTEXEDLG_DELETESERVICE_POLLING_MILLISECOND, m_TextLogWriter );
	CWinThread* lWinThread = AfxBeginThread( CThreadManager::RunThread, ( LPVOID )lDeleteServiceThread, ARRESTEXEAPP_THREAD_PRIORITY, 0, CREATE_SUSPENDED );
	lDeleteServiceThread->SetHandle( lWinThread->m_hThread );
	m_DeleteServiceThreadManager->AddThread( lDeleteServiceThread );
	lWinThread->ResumeThread();
	CAppUtils::SetDeleteServiceThread( lDeleteServiceThread );

	// Create Audit Thread Manager
	if( m_AuditThreadManager != NULL )
	{
		delete m_AuditThreadManager;
	}
	m_AuditThreadManager = new CThreadManager( m_TextLogWriter );

	// Create Audit Threads
	CStringArray* lAuditDirArray = m_AuditOption.GetAuditArray();
	for( int i = 0; i < lAuditDirArray->GetCount(); i++ )
	{
		CString lAuditDir = lAuditDirArray->GetAt( i );
		if( lAuditDir != _T( "" ) )
		{
			// Create Audit Thread
			CAuditThread* lAuditThread = new CAuditThread( lAuditDir, &lExtensionArray, m_AuditOption.GetCompleteDelete(), m_AuditOption.GetUseScan(), m_AuditOption.GetUseStartSearch(), ARRESTEXEDLG_AUDIT_POLLING_MILLISECOND, m_TextLogWriter );
			CWinThread* lWinThread = AfxBeginThread( CThreadManager::RunThread, ( LPVOID )lAuditThread, ARRESTEXEAPP_THREAD_PRIORITY, 0, CREATE_SUSPENDED );
			lAuditThread->SetHandle( lWinThread->m_hThread );
			m_AuditThreadManager->AddThread( lAuditThread );
			lWinThread->ResumeThread();
		}
	}
	CAppUtils::SetRunning( true );

	EndWaitCursor();
}

bool CArrestExeDlg::CheckOption()
{
	CStringArray* lAuditDirArray = m_AuditOption.GetAuditArray();

	bool bNoAuditDir = true;
	for( int i = 0; i < lAuditDirArray->GetCount(); i++ )
	{
		CString lAuditDir = lAuditDirArray->GetAt( i );
		if( lAuditDir != _T( "" ) )
		{
			bNoAuditDir = false;

			// Check Directory Exist
			if( !CFileUtils::IsFileExist( lAuditDir ) )
			{
				CMessageUtils::WMSG( CStringUtils::GetResStr( IDS_WRN_CHECK_DIR_EXIST_HEAD ) + lAuditDir + CStringUtils::GetResStr( IDS_WRN_CHECK_DIR_EXIST_FOOT ) );
				return false;
			}

			// Check Auditable Directory
			if( CFileUtils::IsSpecialFolder( lAuditDir ) )
			{
				CMessageUtils::WMSG( CStringUtils::GetResStr( IDS_WRN_CHECK_DIR_SPC_HEAD ) + lAuditDir + CStringUtils::GetResStr( IDS_WRN_CHECK_DIR_SPC_FOOT ) );
				return false;
			}

			// Check Overlap
			for( int j = 0; j < lAuditDirArray->GetCount(); j++ )
			{
				if( j != i )
				{
					CString lAnotherDir = lAuditDirArray->GetAt( j );
					if( lAnotherDir != _T( "" ) )
					{
						if( CFileUtils::IsPathOverlap( lAuditDir, lAnotherDir ) )
						{
							CMessageUtils::WMSG( CStringUtils::GetResStr( IDS_WRN_CHECK_DIR_OVERLAP_HEAD ) + lAuditDir + CStringUtils::GetResStr( IDS_WRN_CHECK_DIR_OVERLAP_FOOT ) );
							return false;
						}
					}
				}
			}
		}
	}

	// Check Audit Directory Exist
	if( bNoAuditDir )
	{
		CMessageUtils::WMSG( CStringUtils::GetResStr( IDS_WRN_CHECK_NODIR ) );
		return false;
	}

	// Check Extension
	if( m_AuditOption.GetExtensionList() == _T( "" ) )
	{
		CMessageUtils::WMSG( CStringUtils::GetResStr( IDS_WRN_CHECK_EXT_EMPTY ) );
		return false;
	}

	// Disable Control
	EnableControl( false );

	return true;
}

void CArrestExeDlg::EnableControl( bool ipEnable )
{
	LockWindowUpdate();

	m_StatAudit1.EnableWindow( ipEnable );
	m_StatAudit2.EnableWindow( ipEnable );
	m_StatAudit3.EnableWindow( ipEnable );
	m_StatAudit4.EnableWindow( ipEnable );
	m_StatAudit5.EnableWindow( ipEnable );
	m_StatAuditDir1.EnableWindow( ipEnable );
	m_StatAuditDir2.EnableWindow( ipEnable );
	m_StatAuditDir3.EnableWindow( ipEnable );
	m_StatAuditDir4.EnableWindow( ipEnable );
	m_StatAuditDir5.EnableWindow( ipEnable );
	m_BtnRef1.EnableWindow( ipEnable );
	m_BtnRef2.EnableWindow( ipEnable );
	m_BtnRef3.EnableWindow( ipEnable );
	m_BtnRef4.EnableWindow( ipEnable );
	m_BtnRef5.EnableWindow( ipEnable );
	m_BtnClear1.EnableWindow( ipEnable );
	m_BtnClear2.EnableWindow( ipEnable );
	m_BtnClear3.EnableWindow( ipEnable );
	m_BtnClear4.EnableWindow( ipEnable );
	m_BtnClear5.EnableWindow( ipEnable );
	m_StatExtension.EnableWindow( ipEnable );
	m_EditExtension.EnableWindow( ipEnable );
	m_ChkAutorun.EnableWindow( ipEnable );
	m_ChkCompleteDelete.EnableWindow( ipEnable );
	m_ChkRunAsTaskTray.EnableWindow( ipEnable );
	m_ChkUseScan.EnableWindow( m_RDCWUseable ? ipEnable : false );
	m_ChkUseStartSearch.EnableWindow( ipEnable );
	m_ChkTrayOnStart.EnableWindow( ipEnable );
	m_StatGroup.EnableWindow( ipEnable );
	m_StatOption.EnableWindow( ipEnable );

	if( ipEnable )
	{
		m_BtnExecute.SetWindowText( CStringUtils::GetResStr( IDS_BTN_EXEC_ENABLE ) );
	}
	else{
		m_BtnExecute.SetWindowText( CStringUtils::GetResStr( IDS_BTN_EXEC_DISABLE ) );
	}

	UnlockWindowUpdate();
}

void CArrestExeDlg::DoCancel()
{
	BeginWaitCursor();

	MSG lMessage; memset( &lMessage, 0, sizeof( lMessage ) );

	// Cancel Audit Thread
	m_AuditThreadManager->Lock();
	m_AuditThreadManager->CancelAllThread();
	m_AuditThreadManager->Unlock();

	// Wait For Audit Thread End
	while( 1 )
	{
		if( m_AuditThreadManager->GetActiveThreadCount() == 0 )
		{
			break;
		}
		while( ::PeekMessage( &lMessage, m_EditLog.m_hWnd, 0, 0, PM_REMOVE ) )
		{
			::TranslateMessage( &lMessage );
			::DispatchMessage( &lMessage );
		}
		Sleep( ARRESTEXEDLG_CANCEL_POLLING_MILLISECOND );
	}

	// Calcel Delete Service Thread
	m_DeleteServiceThreadManager->Lock();
	m_DeleteServiceThreadManager->CancelAllThread();
	m_DeleteServiceThreadManager->Unlock();
	
	// Wait For Delete Service Thread End
	while( 1 )
	{
		if( m_DeleteServiceThreadManager->GetActiveThreadCount() == 0 )
		{
			break;
		}
		while( ::PeekMessage( &lMessage, m_EditLog.m_hWnd, 0, 0, PM_REMOVE ) )
		{
			::TranslateMessage( &lMessage );
			::DispatchMessage( &lMessage );
		}
		Sleep( ARRESTEXEDLG_CANCEL_POLLING_MILLISECOND );
	}

	// Enable Control
	EnableControl( true );

	CAppUtils::SetRunning( false );

	EndWaitCursor();
}

void CArrestExeDlg::DoExit()
{
	// Cancel
	if( CAppUtils::IsRunning() )
	{
		DoCancel();
	}

	OnCancel();
}

void CArrestExeDlg::OnWindowPosChanging( WINDOWPOS* lpwndpos )
{
	if( !CAppUtils::GetMainWndVisible() )
	{
		lpwndpos->flags &= ~SWP_SHOWWINDOW;
	}
	CDialog::OnWindowPosChanging(lpwndpos);
}

void CArrestExeDlg::ShowTaskTrayMenu()
{
	CPoint lMousePosition;
	::GetCursorPos( &lMousePosition );

	CMenu lTaskTrayMenu;
	lTaskTrayMenu.CreatePopupMenu();
	lTaskTrayMenu.InsertMenu( 0, MF_BYPOSITION, 1, CStringUtils::GetResStr( IDS_BTN_EXEC_ENABLE ) );
	lTaskTrayMenu.InsertMenu( 1, MF_BYPOSITION, 1, CStringUtils::GetResStr( IDS_BTN_EXEC_DISABLE ) );
	lTaskTrayMenu.InsertMenu( 2, MF_BYPOSITION + MF_SEPARATOR );
	lTaskTrayMenu.InsertMenu( 3, MF_BYPOSITION, 2, CStringUtils::GetResStr( IDS_MENU_OPEN ) );
	lTaskTrayMenu.InsertMenu( 4, MF_BYPOSITION, 3, CStringUtils::GetResStr( IDS_MENU_EXIT ) );

	lTaskTrayMenu.EnableMenuItem( 0, MF_BYPOSITION | ( CAppUtils::IsRunning() ? MF_GRAYED : MF_ENABLED ) );
	lTaskTrayMenu.EnableMenuItem( 1, MF_BYPOSITION | ( CAppUtils::IsRunning() ? MF_ENABLED : MF_GRAYED ) );

	SetForegroundWindow();
	BOOL lReturn = lTaskTrayMenu.TrackPopupMenu( TPM_RIGHTALIGN | TPM_BOTTOMALIGN | TPM_RETURNCMD, lMousePosition.x, lMousePosition.y, this );
	if( lReturn != 0 )
	{
		DWORD lParam = MAKELONG( lReturn, 0 );
		switch( lParam )
		{
		case 1:
			DoSwitch();
			break;
		case 2:
			CAppUtils::SetMainWndVisible( true );
			SetForegroundWindow();
			break;
		case 3:
			DoExit();
			break;
		default:
			break;
		}
	}
}

LRESULT CArrestExeDlg::OnTaskTrayMessage( WPARAM wParam, LPARAM lParam )
{
	switch( lParam )
	{
	case WM_LBUTTONUP:
		if( CAppUtils::GetMainWndVisible() )
		{
			ClearWarning();
			CAppUtils::SetMainWndVisible( false );
		}
		else
		{
			CAppUtils::SetMainWndVisible( true );
			SetForegroundWindow();
		}
		break;
	case WM_RBUTTONUP:
		ShowTaskTrayMenu();
		return 0;
	default:
		break;
	}
	return 0;
}

void CArrestExeDlg::ClearWarning()
{
	if( CAppUtils::GetTaskTrayIcon() == CAppUtils::TASKTRAY_WARN )
	{
		CAppUtils::SetTaskTrayIcon( CAppUtils::IsRunning() ? CAppUtils::TASKTRAY_RUN : CAppUtils::TASKTRAY_STOP );
	}
}

void CArrestExeDlg::OnBnClickedBtnTasktray()
{
	ClearWarning();
	CAppUtils::SetMainWndVisible( false );
}

LRESULT CArrestExeDlg::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	if( message == WM_CTLCOLORSTATIC )
	{
		HWND lWindowHandle = ( HWND )lParam;
		if( lWindowHandle == m_EditLog.m_hWnd )
		{
			return ( LRESULT )GetSysColorBrush( COLOR_WINDOW );
		}
	}
	return CDialog::WindowProc(message, wParam, lParam);
}

void CArrestExeDlg::UpdateStartUpMenuEnable()
{
	bool bLinkExist = CFileUtils::IsFileExist( CAppUtils::GetStartupLinkPath() );
	CMenu *lSysMenu = GetSystemMenu( NULL );
	lSysMenu->EnableMenuItem( 0, MF_BYPOSITION | ( bLinkExist ? MF_GRAYED : MF_ENABLED ) );
	lSysMenu->EnableMenuItem( 1, MF_BYPOSITION | ( bLinkExist ? MF_ENABLED : MF_GRAYED ) );
}

void CArrestExeDlg::OnMenuSysStartUp( bool ipAdd )
{
	CString lLinkPath = CAppUtils::GetStartupLinkPath();

	// Delete Link
	if( CFileUtils::IsFileExist( lLinkPath ) )
	{
		if( !CFileUtils::DeleteFile( lLinkPath ) )
		{
			CMessageUtils::EMSG( CStringUtils::GetResStr( IDS_ERR_SHORTCUT_DEL ) );
			return;
		}
	}

	// Create Link
	if( ipAdd )
	{
		if( !SUCCEEDED( CFileUtils::CreateLink( CFileUtils::GetModulePath(), lLinkPath, CStringUtils::GetResStr( IDS_ETC_LINKTITLE ) ) ) )
		{
			CMessageUtils::EMSG( CStringUtils::GetResStr( IDS_ERR_SHORTCUT_ADD ) );
			return;
		}
		if( !CAppUtils::IsRunning() )
		{
			m_ChkAutorun.SetCheck( BST_CHECKED );
			m_ChkRunAsTaskTray.SetCheck( BST_CHECKED );
		}
		CMessageUtils::IMSG( CStringUtils::GetResStr( IDS_INFO_SHORCUT_ADD ) );
	}
	else
	{
		if( !CAppUtils::IsRunning() )
		{
			m_ChkAutorun.SetCheck( BST_UNCHECKED );
			m_ChkRunAsTaskTray.SetCheck( BST_UNCHECKED );
		}
		CMessageUtils::IMSG( CStringUtils::GetResStr(IDS_INFO_SHORCUT_DEL  ) );
	}

	// Update System Menu
	UpdateStartUpMenuEnable();
}

void CArrestExeDlg::OnMenuSysAddShowHP()
{
	ShellExecute( NULL, _T( "open" ), ARRESTEXEDLG_SUPPORT_URL, NULL, NULL, SW_SHOWNORMAL );
}

