//	ArrestExe - File Audit Utility
//	Copyright (C) 2002-2006 AnonyMouse
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once

#ifndef __ARRESTEXEDLG_H__
#define __ARRESTEXEDLG_H__

#include "afxwin.h"
#include "ThreadManager.h"
#include "AuditThread.h"
#include "TextLogWriter.h"
#include "AuditOption.h"
#include "BtnSt.h"

#define ARRESTEXEDLG_SYSMENU_ADD_STARTUP ( WM_USER + 1000 )
#define ARRESTEXEDLG_SYSMENU_DEL_STARTUP ( WM_USER + 1001 )
#define ARRESTEXEDLG_SYSMENU_SHOW_HP ( WM_USER + 1002 )

#define ARRESTEXEDLG_DELETESERVICE_POLLING_MILLISECOND 100
#define ARRESTEXEDLG_AUDIT_POLLING_MILLISECOND 200
#define ARRESTEXEDLG_CANCEL_POLLING_MILLISECOND 100

#define ARRESTEXEDLG_SUPPORT_URL _T( "http://red.sakura.ne.jp/~anonymouse/" )

class CArrestExeDlg : public CDialog
{
// Construction
public:
	CArrestExeDlg( CWnd* pParent = NULL );
	~CArrestExeDlg();
	enum { IDD = IDD_ARRESTEXE_DIALOG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);

// Private Methods
private:
	void InitTrayIcon();

private:
	void LoadOption();
	void SaveOption();

private:
	void DoSwitch( bool ipTrayOnStart = false );
	void DoAudit();
	void DoCancel();
	void DoExit();
	bool CheckOption();
	void EnableControl( bool ipEnable );
	void ValidateExtensionList();
	bool IsAuditDirOverlap( CStatic* ipStatSelf, CStatic* ipStatTarget, CString ipDirPath );

private:
	void SelectAuditDir( CStatic* ipAuditDirLabel );
	void ClearAuditDir( CStatic* ipAuditDirLabel );

private:
	void ShowTaskTrayMenu();
	void UpdateStartUpMenuEnable();
	void OnMenuSysStartUp( bool ipAdd );
	void OnMenuSysAddShowHP();
	void ClearWarning();

// Private Variables
private:
	CAuditOption m_AuditOption;
	CTextLogWriter* m_TextLogWriter;
	CThreadManager* m_AuditThreadManager;
	CThreadManager* m_DeleteServiceThreadManager;
	BOOL m_RDCWUseable;

// Override
protected:
	HICON m_hIcon;
	HICON m_hIcon16;
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();

// DDX Variables
public:
	CEdit m_EditLog;
	CEdit m_EditExtension;
	CStatic m_StatAudit1;
	CStatic m_StatAudit2;
	CStatic m_StatAudit3;
	CStatic m_StatAudit4;
	CStatic m_StatAudit5;
	CStatic m_StatAuditDir1;
	CStatic m_StatAuditDir2;
	CStatic m_StatAuditDir3;
	CStatic m_StatAuditDir4;
	CStatic m_StatAuditDir5;
	CButton m_BtnRef1;
	CButton m_BtnRef2;
	CButton m_BtnRef3;
	CButton m_BtnRef4;
	CButton m_BtnRef5;
	CButton m_BtnClear1;
	CButton m_BtnClear2;
	CButton m_BtnClear3;
	CButton m_BtnClear4;
	CButton m_BtnClear5;
	CStatic m_StatExtension;
	CButton m_ChkAutorun;
	CButton m_ChkCompleteDelete;
	CButton m_ChkRunAsTaskTray;
	CButton m_ChkUseScan;
	CButton m_ChkUseStartSearch;
	CButton m_ChkTrayOnStart;
	CButton m_BtnExecute;
	CButton m_BtnExit;
	CButton m_BtnLogClear;
	CStatic m_StatGroup;
	CStatic m_StatOption;
	CButtonST m_BtnTaskTray;

// Message Map
DECLARE_MESSAGE_MAP()
private:
	afx_msg void OnBnClickedBtnRef1();
	afx_msg void OnBnClickedBtnRef2();
	afx_msg void OnBnClickedBtnRef3();
	afx_msg void OnBnClickedBtnRef4();
	afx_msg void OnBnClickedBtnRef5();
	afx_msg void OnBnClickedBtnClear1();
	afx_msg void OnBnClickedBtnClear2();
	afx_msg void OnBnClickedBtnClear3();
	afx_msg void OnBnClickedBtnClear4();
	afx_msg void OnBnClickedBtnClear5();
	afx_msg void OnBnClickedChkAutorun();
	afx_msg void OnBnClickedChkCompleteDelete();
	afx_msg void OnBnClickedCancel();
	afx_msg void OnEnKillfocusEditExtension();
	afx_msg void OnBnClickedBtnExecute();
	afx_msg void OnBnClickedChkRunAsTasktray();
	afx_msg void OnBnClickedBtnLogclear();
	afx_msg void OnWindowPosChanging( WINDOWPOS* lpwndpos );
	afx_msg LRESULT OnTaskTrayMessage( WPARAM wParam, LPARAM lParam );
	afx_msg void OnBnClickedBtnExit();
	afx_msg void OnBnClickedBtnTasktray();
	afx_msg void OnBnClickedChkUseScan();
	afx_msg void OnBnClickedChkUseStartSearch();
	afx_msg void OnBnClickedChkTrayOnStart();
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
};

#endif // __ARRESTEXEDLG_H__
