//	ArrestExe - File Audit Utility
//	Copyright (C) 2002-2006 AnonyMouse
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Project Includes:
#include "Stdafx.h"
#include "ArrestExe.h"
#include "AuditOption.h"

// General includes:

// Utility includes:
#include "FileUtils.h"

// Debug Codes:
#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CAuditOption::CAuditOption()
{
	m_AutoRun = false;
	m_CompleteDelete = false;
	m_UseScan = false;

	Load();
}

CAuditOption::~CAuditOption()
{
}

CStringArray* CAuditOption::GetAuditArray()
{
	return &m_AuditArray;
}

CString CAuditOption::GetExtensionList()
{
	return m_ExtensionList;
}

void CAuditOption::SetExtensionList( CString ipExtensionList )
{
	m_ExtensionList = ipExtensionList;
}

bool CAuditOption::GetAutoRun()
{
	return m_AutoRun;
}

bool CAuditOption::GetCompleteDelete()
{
	return m_CompleteDelete;
}

void CAuditOption::SetAutoRun( bool ipAutoRun )
{
	m_AutoRun = ipAutoRun;
}

void CAuditOption::SetCompleteDelete( bool ipCompleteDelete )
{
	m_CompleteDelete = ipCompleteDelete;
}

bool CAuditOption::GetRunAsTaskTray()
{
	return m_RunAsTaskTray;
}

void CAuditOption::SetRunAsTaskTray( bool ipTaskTray )
{
	m_RunAsTaskTray = ipTaskTray;
}

bool CAuditOption::GetUseScan()
{
	return m_UseScan;
}

void CAuditOption::SetUseScan( bool ipUseScan )
{
	m_UseScan = ipUseScan;
}

bool CAuditOption::GetUseStartSearch()
{
	return m_UseStartSearch;
}

void CAuditOption::SetUseStartSearch( bool ipUseStartSearch )
{
	m_UseStartSearch = ipUseStartSearch;
}

bool CAuditOption::GetTrayOnStart()
{
	return m_TrayOnStart;
}

void CAuditOption::SetTrayOnStart( bool ipTrayOnStart )
{
	m_TrayOnStart = ipTrayOnStart;
}

void CAuditOption::Load()
{
	// Audit Folders
	m_AuditArray.Add( AfxGetApp()->GetProfileString( _T( "AuditOption" ), _T( "AuditFolder0" ), _T( "" ) ) );
	m_AuditArray.Add( AfxGetApp()->GetProfileString( _T( "AuditOption" ), _T( "AuditFolder1" ), _T( "" ) ) );
	m_AuditArray.Add( AfxGetApp()->GetProfileString( _T( "AuditOption" ), _T( "AuditFolder2" ), _T( "" ) ) );
	m_AuditArray.Add( AfxGetApp()->GetProfileString( _T( "AuditOption" ), _T( "AuditFolder3" ), _T( "" ) ) );
	m_AuditArray.Add( AfxGetApp()->GetProfileString( _T( "AuditOption" ), _T( "AuditFolder4" ), _T( "" ) ) );
	
	// Extensions
	m_ExtensionList = GetValidExtensionList( AfxGetApp()->GetProfileString( _T( "AuditOption" ), _T( "ExtensionList" ), _T( "exe,scb,scr,bat,com,cmd,pif,wsf,wsh,vbs,vbe,js,lnk" ) ) );

	// Complete Delete
	m_CompleteDelete = ( AfxGetApp()->GetProfileInt( _T( "AuditOption" ), _T( "CompleteDelete" ), 1 ) == 1 );

	// Run As TaskTray
	m_RunAsTaskTray = ( AfxGetApp()->GetProfileInt( _T( "AuditOption" ), _T( "RunAsTaskTray" ), 0 ) == 1 );

	// Auto Run
	m_AutoRun = ( AfxGetApp()->GetProfileInt( _T( "AuditOption" ), _T( "AutoRun" ), 0 ) == 1 );

	// Use Scan
	m_UseScan = ( AfxGetApp()->GetProfileInt( _T( "AuditOption" ), _T( "UseScan" ), 0 ) == 1 );

	// Use Start Search
	m_UseStartSearch = ( AfxGetApp()->GetProfileInt( _T( "AuditOption" ), _T( "UseStartSearch" ), 0 ) == 1 );

	// Tray On Start
	m_TrayOnStart = ( AfxGetApp()->GetProfileInt( _T( "AuditOption" ), _T( "TrayOnStart" ), 0 ) == 1 );
}

void CAuditOption::Save()
{
	// Audit Folders
	for( int i = 0; i < m_AuditArray.GetCount(); i++ )
	{
		CString lKey = _T( "" );
		lKey.Format( _T( "%s%d" ), _T( "AuditFolder" ), i );
		AfxGetApp()->WriteProfileString( _T( "AuditOption" ), lKey, m_AuditArray.GetAt( i ) );
	}

	// Extensions
	AfxGetApp()->WriteProfileString( _T( "AuditOption" ), _T( "ExtensionList" ), m_ExtensionList );

	// Complete Delete
	AfxGetApp()->WriteProfileInt( _T( "AuditOption" ), _T( "CompleteDelete" ), m_CompleteDelete ? 1 : 0 );

	// Run As Task Tray
	AfxGetApp()->WriteProfileInt( _T( "AuditOption" ), _T( "RunAsTaskTray" ), m_RunAsTaskTray ? 1 : 0 );

	// Auto Run
	AfxGetApp()->WriteProfileInt( _T( "AuditOption" ), _T( "AutoRun" ), m_AutoRun ? 1 : 0 );

	// Use Scan
	AfxGetApp()->WriteProfileInt( _T( "AuditOption" ), _T( "UseScan" ), m_UseScan ? 1 : 0 );

	// Use Start Search
	AfxGetApp()->WriteProfileInt( _T( "AuditOption" ), _T( "UseStartSearch" ), m_UseStartSearch ? 1 : 0 );

	// Tray On Start
	AfxGetApp()->WriteProfileInt( _T( "AuditOption" ), _T( "TrayOnStart" ), m_TrayOnStart ? 1 : 0 );
}

CString CAuditOption::GetValidExtensionList( CString ipExtensionList )
{
	CStringArray lExtensionArray;
	int lPosition = 0;
	CString lExtension = ipExtensionList.Tokenize( AUDITOPTION_EXTENSION_SEPARATER, lPosition );
	while( lExtension != _T( "" ) )
	{
		lExtension.Trim( _T( " \t\r\n" ) );
		if( lExtension != _T( "" ) )
		{
			// Pickup Useable Characters
			CString lUsableExtension = _T( "" );
			for( int i = 0; i < lExtension.GetLength(); i++ )
			{
				TCHAR lChar = lExtension.GetAt( i );
				if( CFileUtils::IsUsableChar( lChar ) )
				{
					lUsableExtension += lChar;
				}
			}
			// Dupulication Check
			if( lUsableExtension != _T( "" ) )
			{
				bool bFound = false;
				for( int i = 0; i < lExtensionArray.GetCount(); i++ )
				{
					if( lExtensionArray.GetAt( i ) == lUsableExtension )
					{
						bFound = true;
						break;
					}
				}
				if( !bFound )
				{
					lExtensionArray.Add( lUsableExtension );
				}
			}
		}
		lExtension = ipExtensionList.Tokenize( AUDITOPTION_EXTENSION_SEPARATER, lPosition );
	}

	// CreateList From Array
	CString lValidExtension = _T( "" );
	for( int i = 0; i < lExtensionArray.GetCount(); i++ )
	{
		lValidExtension += lExtensionArray.GetAt( i );
		lValidExtension += AUDITOPTION_EXTENSION_SEPARATER;
	}

	// Remove Last Separater
	if( lValidExtension != _T( "" ) )
	{
		lValidExtension = lValidExtension.Left( lValidExtension.GetLength() - 1  );
	}

	return lValidExtension;
}


