//	ArrestExe - File Audit Utility
//	Copyright (C) 2002-2006 AnonyMouse
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Project Includes:
#include "Stdafx.h"
#include "ArrestExe.h"
#include "DeleteServiceThread.h"
#include "DeleteThread.h"
#include "ArrestExeDlg.h"

// General includes:

// Utility includes:
#include "FileUtils.h"
#include "StringUtils.h"
#include "CommonProcUtils.h"

// Debug Codes:
#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CDeleteServiceThread::CDeleteServiceThread( bool ipCompleteDelete, int ipPollingMilliSecond, CLogWriter* ipLogWriter ):CThreadHandler( ipPollingMilliSecond, ipLogWriter )
{
	m_CompleteDelete = ipCompleteDelete;
	m_ThreadManager = new CThreadManager( ipLogWriter );
}

CDeleteServiceThread::~CDeleteServiceThread()
{
	delete m_ThreadManager;
}

void CDeleteServiceThread::OnEndThread()
{
	// Cancel All Thread
	m_ThreadManager->Lock();
	m_ThreadManager->CancelAllThread();
	m_ThreadManager->Unlock();

	// Wait For Cancel Complete
	while( 1 )
	{
		if( m_ThreadManager->GetActiveThreadCount() == 0 )
		{
			break;
		}
		Sleep( DELETESERVICTHREAD_CANCEL_POLLING_MILLISECOND );
	}
}

UINT CDeleteServiceThread::RunImpl()
{
	while( 1 )
	{
		// Check Cancel
		if( IsCanceled() )
		{
			return CThreadHandler::THREAD_DONE;
		}

		// Get FileName From Que
		CString lFileName = _T( "" );
		m_QueCriticalSection.Lock();
		if( m_FilePathQue.GetCount() != 0 )
		{
			lFileName = m_FilePathQue.GetAt( 0 );
			m_FilePathQue.RemoveAt ( 0 );
		}
		m_QueCriticalSection.Unlock();
		if( lFileName == _T( "" ) )
		{
			break;
		}
		
		// Check File Exist
		if( CFileUtils::IsFileExist( lFileName ) )
		{
			// Lock Thread Manager
			if( !m_ThreadManager->Lock() )
			{
				return CThreadHandler::THREAD_DONE;
			}
			
			// Check Thread Exist
			bool bThreadFound = false;
			int lThreadCount = m_ThreadManager->GetActiveThreadCount();
			for( int i = 0; i < lThreadCount; i++ )
			{
				if( ( ( CDeleteThread* )m_ThreadManager->GetThread( i ) )->GetFilePath() == lFileName )
				{
					bThreadFound = true;
					break;
				}
			}
			
			// If File Locked, Create Arrest Thread
			bool bLocked = false;
			if( !bThreadFound )
			{
				if( CFileUtils::IsFileLocked( lFileName ) )
				{
					bLocked = true;
					CDeleteThread* lArrestThread = new CDeleteThread( lFileName, m_CompleteDelete, DELETESERVICTHREAD_ARREST_POLLING_MILLISECOND, m_LogWriter );
					CWinThread* lWinThread = AfxBeginThread( CThreadManager::RunThread, ( LPVOID )lArrestThread, ARRESTEXEAPP_THREAD_PRIORITY, 0, CREATE_SUSPENDED );
					lArrestThread->SetHandle( lWinThread->m_hThread );
					m_ThreadManager->AddThread( lArrestThread );
					lWinThread->ResumeThread();
				}
			}
			
			// UnLock Thread Manager
			m_ThreadManager->Unlock();
			
			// Delete File
			if( !bThreadFound && !bLocked )
			{
				CCommonProcUtils::DoDeleteFile( lFileName, m_CompleteDelete, m_LogWriter );
			}

		}	// If File Exist
	}
	return CThreadHandler::THREAD_RESUME;
}

void CDeleteServiceThread::RequestDelete( CString ipFilePath )
{
	// Lock Que
	m_QueCriticalSection.Lock();
	
	// Find Same Path
	bool bFound = false;
	int lFileCount = ( int )m_FilePathQue.GetCount();
	for( int i = 0; i < lFileCount; i++ )
	{
		if( m_FilePathQue.GetAt( i ) == ipFilePath )
		{
			bFound = true;
			break;
		}
	}
	
	// If Not Found, Add To Que
	if( !bFound )
	{
		m_FilePathQue.Add( ipFilePath );
	}
	
	// Unlock Que
	m_QueCriticalSection.Unlock();
}

int CDeleteServiceThread::GetQueCount()
{
	return ( int )m_FilePathQue.GetCount();
}

