//	ArrestExe - File Audit Utility
//	Copyright (C) 2002-2006 AnonyMouse
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Project Includes:
#include "Stdafx.h"
#include "ArrestExe.h"
#include "DeleteThread.h"

// General includes:

// Utility includes:
#include "FileUtils.h"
#include "StringUtils.h"
#include "CommonProcUtils.h"

// Debug Codes:
#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CDeleteThread::CDeleteThread( CString ipFilePath, bool ipCompleteDelete, int ipPollingMilliSecond, CLogWriter* ipLogWriter ):CThreadHandler( ipPollingMilliSecond, ipLogWriter )
{
	m_FilePath = ipFilePath;
	m_CompleteDelete = ipCompleteDelete;
	m_Resumed = false;
}

CDeleteThread::~CDeleteThread()
{
}

UINT CDeleteThread::RunImpl()
{
	// Check File Exist
	if( !CFileUtils::IsFileExist( m_FilePath ) )
	{
		return CThreadHandler::THREAD_DONE;
	}

	// Check File Locked
	if( CFileUtils::IsFileLocked( m_FilePath ) )
	{
		m_Resumed = true;
		return CThreadHandler::THREAD_RESUME;
	}

	// Wait For Another Process's Rename
	if( m_Resumed )
	{
		Sleep( DELETETHREAD_WAIT_FOR_ANOTHRE_PROCESS_RENAME_MILLISECOND );
		if( !CFileUtils::IsFileExist( m_FilePath ) )
		{
			return CThreadHandler::THREAD_DONE;
		}
	}

	// Delete File
	bool bReturn = CCommonProcUtils::DoDeleteFile( m_FilePath, m_CompleteDelete, m_LogWriter );

	return bReturn ? CThreadHandler::THREAD_DONE : CThreadHandler::THREAD_ERROR;
}

CString CDeleteThread::GetFilePath()
{
	return m_FilePath;
}

