//	ArrestExe - File Audit Utility
//	Copyright (C) 2002-2006 AnonyMouse
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Project Includes:
#include "Stdafx.h"
#include "ArrestExe.h"
#include "FileUtils.h"

// General includes:
#include <shlwapi.h>

// Utility includes:

// Debug Codes:
#ifdef _DEBUG
#define new DEBUG_NEW
#endif

bool CFileUtils::IsNetwork( CString ipPath )
{
	if( ipPath.GetLength() >= 2 && ipPath.GetAt( 0 ) == _T( '\\' ) && ipPath.GetAt( 1 ) == _T( '\\' ) )
	{
		return true;
	}
	return ( GetDriveType( GetDrive( ipPath ) ) == DRIVE_REMOTE );
}

bool CFileUtils::IsDirectory( CString ipFilePath )
{
	DWORD lFileAttribute = GetFileAttributes( ipFilePath );
	if( lFileAttribute == 0xFFFFFFFF )
	{
		return false;
	}
	return ( ( lFileAttribute & FILE_ATTRIBUTE_DIRECTORY ) == FILE_ATTRIBUTE_DIRECTORY );
}

bool CFileUtils::IsFileExist( CString ipFilePath )
{
	return ( GetFileAttributes( ipFilePath ) != 0xFFFFFFFF );
}

bool CFileUtils::IsFileLocked( CString ipFilePath )
{
	HANDLE lHandle = CreateFile( ipFilePath, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL , NULL );
	DWORD lErrCode = GetLastError();
	if( lHandle != INVALID_HANDLE_VALUE )
	{
		CloseHandle( lHandle );
		return false;
	}
	return ( lErrCode == FILE_UTILS_ANOTHER_PROCESS_USING_ERROR );
}

bool CFileUtils::TrashFile( CString ipFilePath )
{
	SHFILEOPSTRUCT lFileOption;
	CString lFilePath = ipFilePath + _T( '\0' );

	memset( &lFileOption, NULL, sizeof( SHFILEOPSTRUCT ) );
	lFileOption.hwnd = NULL;
	lFileOption.wFunc = FO_DELETE;
	lFileOption.pFrom = lFilePath;
	lFileOption.pTo   = NULL;
	lFileOption.fFlags = FOF_ALLOWUNDO | FOF_SILENT | FOF_NOCONFIRMATION | FOF_FILESONLY | FOF_NOERRORUI;
	return ( SHFileOperation ( &lFileOption ) == 0 );
}

bool CFileUtils::DeleteFile( CString ipFilePath )
{
	::SetFileAttributes( ipFilePath, FILE_ATTRIBUTE_NORMAL );
	return ( ::DeleteFile( ipFilePath ) == TRUE );
}

bool CFileUtils::IsUsableChar( TCHAR ipChar )
{
	UINT lReturn = GCT_INVALID;
	lReturn = PathGetCharType( ipChar );
	return ( ( lReturn & GCT_LFNCHAR ) || ( lReturn & GCT_SHORTCHAR ) );
}

CString CFileUtils::GetModulePath()
{
	TCHAR lPath[ FILE_UTILS_MAX_PATH ];
	::GetModuleFileName( NULL, lPath, sizeof( lPath ) );
	CString lModulePath = lPath;
	return lModulePath;
}

CString CFileUtils::GetFolderPath( CString ipFilePath )
{
	TCHAR lDrive[ FILE_UTILS_MAX_PATH ];
	TCHAR lDirectory[ FILE_UTILS_MAX_PATH ];
	TCHAR lFile[ FILE_UTILS_MAX_PATH ];
	TCHAR lExt[ FILE_UTILS_MAX_PATH ];
	_tsplitpath( ipFilePath, lDrive, lDirectory, lFile, lExt );

	CString lFolderPath = lDrive;
	lFolderPath += lDirectory;

	return lFolderPath;
}

CString CFileUtils::GetFileExt( CString ipFilePath )
{
	TCHAR lDrive[ FILE_UTILS_MAX_PATH ];
	TCHAR lDirectory[ FILE_UTILS_MAX_PATH ];
	TCHAR lFile[ FILE_UTILS_MAX_PATH ];
	TCHAR lExt[ FILE_UTILS_MAX_PATH ];

	_tsplitpath( ipFilePath, lDrive, lDirectory, lFile, lExt );

	CString lReturn = lExt;
	if( lReturn.GetLength() > 0 && lReturn.GetAt( 0 ) == _T( '.' ) )
	{
		lReturn = lReturn.Right( lReturn.GetLength() - 1 );
	}

	return lReturn;
}

CString CFileUtils::GetDrive( CString ipFilePath )
{
	TCHAR lDrive[ FILE_UTILS_MAX_PATH ];
	TCHAR lDirectory[ FILE_UTILS_MAX_PATH ];
	TCHAR lFile[ FILE_UTILS_MAX_PATH ];
	TCHAR lExt[ FILE_UTILS_MAX_PATH ];

	_tsplitpath( ipFilePath, lDrive, lDirectory, lFile, lExt );

	CString lReturn = lDrive;

	return lReturn;
}

CString CFileUtils::GetSpecialFolderPath( int ipCSIDL, HWND ipWindowHandle )
{
	IMalloc *lMalloc;
	ITEMIDLIST *lItemIDList;
	TCHAR lPathBuffer[ FILE_UTILS_MAX_PATH ];
	memset( lPathBuffer, 0, sizeof( lPathBuffer ) );
	CString lReturn = _T( "" );
	
	if( SHGetMalloc( &lMalloc ) == NOERROR )
	{
		if( SHGetSpecialFolderLocation( ipWindowHandle, ipCSIDL, &lItemIDList ) == NOERROR )
		{
			if( SHGetPathFromIDList( lItemIDList, lPathBuffer ) )
			{
				lMalloc->Free( lItemIDList );
				lMalloc->Release();
				lReturn = lPathBuffer;
				return lReturn;
			}
			lMalloc->Free( lItemIDList );
		}
		lMalloc->Release();
	}
	return lReturn;
}

bool CFileUtils::IsPathEqual( CString ipPathA, CString ipPathB )
{
	ipPathA.TrimRight( _T( "\\" ) );
	ipPathA.MakeLower();
	ipPathB.TrimRight( _T( "\\" ) );
	ipPathB.MakeLower();
	return( ipPathA == ipPathB );
}

bool CFileUtils::IsPathOverlap( CString ipPathA, CString ipPathB )
{
	ipPathA.TrimRight( _T( "\\" ) );
	ipPathA.MakeLower();
	ipPathA += _T( "\\" );
	ipPathB.TrimRight( _T( "\\" ) );
	ipPathB.MakeLower();
	ipPathB += _T( "\\" );
	return ( ipPathA.Find( ipPathB ) == 0 || ipPathB.Find( ipPathA ) == 0 );
}

bool CFileUtils::IsSpecialFolder( CString ipFilePath )
{
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_ADMINTOOLS ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_ALTSTARTUP ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_BITBUCKET ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_CDBURN_AREA ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_ADMINTOOLS ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_ALTSTARTUP ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_APPDATA ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_DESKTOPDIRECTORY ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_DOCUMENTS ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_FAVORITES ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_MUSIC ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_PICTURES ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_PROGRAMS ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_STARTMENU ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_STARTUP ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_TEMPLATES ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COMMON_VIDEO ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_CONTROLS ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_COOKIES ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_DRIVES ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_FAVORITES ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_FONTS ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_HISTORY ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_INTERNET_CACHE ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_LOCAL_APPDATA ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_MYDOCUMENTS ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_NETHOOD ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_NETWORK ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_PRINTERS ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_PRINTHOOD ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_PROFILE ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_PROGRAM_FILES ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_PROGRAM_FILES_COMMON ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_PROGRAMS ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_RECENT ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_SENDTO ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_STARTMENU ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_STARTUP ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_SYSTEM ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_TEMPLATES ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetSpecialFolderPath( CSIDL_WINDOWS ) ) ) return true;
	if( IsPathEqual( ipFilePath, GetDrive( GetSpecialFolderPath( CSIDL_SYSTEM ) ) ) ) return true;
	if( ipFilePath.GetLength() > 1 && ipFilePath.GetAt( 0 ) == _T( '\\' ) && ipFilePath.GetAt( 1 ) != _T( '\\' ) ) return true;
	if( ipFilePath.GetLength() > 0 && ipFilePath.GetAt( 0 ) == _T( '.' ) ) return true;

	// Older Version Check
	if( GetSpecialFolderPath( CSIDL_PROGRAM_FILES ) == _T( "" ) )
	{
		if( IsPathEqual( ipFilePath, CFileUtils::GetSystemDirectory() ) ) return true;
		if( IsPathEqual( ipFilePath, CFileUtils::GetWindowsDirectory() ) ) return true;
		if( IsPathEqual( ipFilePath, CFileUtils::GetProgramFilesDirectory() ) ) return true;
		if( IsPathEqual( ipFilePath, GetDrive( GetWindowsDirectory() ) ) ) return true;
	}

	return false;
}

HRESULT CFileUtils::CreateLink( CString ipSrcFilePath, CString ipLinkFilePath, CString ipDescription ) 
{ 
    HRESULT lResult;
    IShellLink* lShellLink;
 
    lResult = CoCreateInstance( CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, ( LPVOID* ) &lShellLink );
    if( SUCCEEDED ( lResult ) )
    { 
        IPersistFile* lPersistFile; 

		lShellLink->SetPath( ipSrcFilePath ); 
        lShellLink->SetDescription( ipDescription ); 
 
        lResult = lShellLink->QueryInterface( IID_IPersistFile, ( LPVOID * ) &lPersistFile ); 
        if( SUCCEEDED ( lResult ) ) 
        {
#ifdef _UNICODE
            lResult = lPersistFile->Save( ipLinkFilePath, TRUE ); 
#else _MBCS
			WCHAR lWideCharBuffer[ FILE_UTILS_MAX_PATH ]; 
			MultiByteToWideChar( CP_ACP, 0, ipLinkFilePath, -1, lWideCharBuffer, FILE_UTILS_MAX_PATH ); 
			lResult = lPersistFile->Save( lWideCharBuffer, TRUE ); 
#endif
            lPersistFile->Release(); 
        } 
        lShellLink->Release(); 
    } 
    return lResult; 
}

CString CFileUtils::GetSystemDirectory()
{
	TCHAR lPath[ FILE_UTILS_MAX_PATH ];
	::GetSystemDirectory( lPath, FILE_UTILS_MAX_PATH );
	return lPath;
}

CString CFileUtils::GetWindowsDirectory()
{
	TCHAR lPath[ FILE_UTILS_MAX_PATH ];
	::GetWindowsDirectory( lPath, FILE_UTILS_MAX_PATH );
	return lPath;
}

CString CFileUtils::GetProgramFilesDirectory()
{
	CString lRetPath = _T( "" );
	HKEY lCurrentVersion = NULL;
	LONG lRetCode = 0;

	lRetCode = RegOpenKeyEx( HKEY_LOCAL_MACHINE, _T( "SOFTWARE\\Microsoft\\Windows\\CurrentVersion" ), 0, KEY_READ, &lCurrentVersion );
	if( lRetCode != ERROR_SUCCESS )
	{
		return lRetPath;
	}

	do
	{
		DWORD lRegType;
		TCHAR lBuffer[ FILE_UTILS_MAX_PATH ];
		DWORD lBufferSize = sizeof( lBuffer );
		lRetCode = RegQueryValueEx( lCurrentVersion, _T( "ProgramFilesDir" ), NULL, &lRegType, ( LPBYTE )&lBuffer, &lBufferSize );
		if( lRetCode != ERROR_SUCCESS )
		{
			break;
		}
		lRetPath = lBuffer;

		break;

	} while( true );

	RegCloseKey( lCurrentVersion );

	return lRetPath;
}
