//	ArrestExe - File Audit Utility
//	Copyright (C) 2002-2006 AnonyMouse
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Project Includes:
#include "Stdafx.h"
#include "ArrestExe.h"
#include "ThreadHandler.h"

// General includes:

// Utility includes:

// Debug Codes:
#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CThreadHandler::CThreadHandler( int ipPollingMilliSecond, CLogWriter* ipLogWriter ):CLogHandler( ipLogWriter )
{
	m_PollingMilliSecond = ipPollingMilliSecond;
	m_Canceled = false;
	m_Thread = NULL;
}

CThreadHandler::~CThreadHandler()
{
	if( m_Thread != NULL )
	{
		::CloseHandle( m_Thread );
	}
}

void CThreadHandler::SetHandle( HANDLE ipThreadHandle )
{
	if( !::DuplicateHandle( GetCurrentProcess(), ipThreadHandle, GetCurrentProcess(), &m_Thread, 0, FALSE, DUPLICATE_SAME_ACCESS ) )
	{
		m_Thread = NULL;
	}
}

HANDLE CThreadHandler::GetHandle()
{
	return m_Thread;
}

bool CThreadHandler::IsCanceled()
{
	return m_Canceled;
}

UINT CThreadHandler::Run()
{
	UINT lReturn = 0;

	while( 1 )
	{
		if( m_Thread == NULL )
		{
			break;
		}
		if( m_Canceled )
		{
			break;
		}
		lReturn = RunImpl();
		if( lReturn == CThreadHandler::THREAD_DONE || lReturn == CThreadHandler::THREAD_ERROR )
		{
			break;
		}
		Sleep( m_PollingMilliSecond );
	}
	OnEndThread();

	return lReturn;
}

void CThreadHandler::Cancel()
{
	m_Canceled = true;
}

void CThreadHandler::OnEndThread()
{
}
