//	ArrestExe - File Audit Utility
//	Copyright (C) 2002-2006 AnonyMouse
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Project Includes:
#include "Stdafx.h"
#include "ArrestExe.h"
#include "ThreadManager.h"

// General includes:

// Utility includes:

// Debug Codes:
#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CThreadManager::CThreadManager( CLogWriter* ipLogWriter ):CLogHandler( ipLogWriter )
{
	m_Canceled = false;
}

CThreadManager::~CThreadManager()
{
}

bool CThreadManager::Lock()
{
	m_CriticalSection.Lock();
	if( m_Canceled )
	{
		m_CriticalSection.Unlock();
		return false;
	}
	return true;
}

void CThreadManager::Unlock()
{
	m_CriticalSection.Unlock();
}

void CThreadManager::CancelAllThread()
// Should Be Used After Lock
{
	m_Canceled = true;
	for( int i = 0; i < m_ThreadArray.GetCount(); i++ )
	{
		CThreadHandler* lThreadHandler = ( CThreadHandler* )m_ThreadArray.GetAt( i );
		lThreadHandler->Cancel();
	}
}

CThreadHandler* CThreadManager::GetThread( int ipIndex )
// Should Be Used After Lock
{
	return ( CThreadHandler* )( m_ThreadArray.GetAt( ipIndex ) );
}

void CThreadManager::AddThread( CThreadHandler* ipThread )
// Should Be Used After Lock
{
	// Add Thread To Array
	m_ThreadArray.Add( ipThread );

	// Create Watch Thread Parameter
	SWatchThreadParam* lWatchThreadParam = new SWatchThreadParam;
	memset( lWatchThreadParam, 0, sizeof( lWatchThreadParam ) );
	lWatchThreadParam->m_ThreadManager = this;
	lWatchThreadParam->m_ThreadHandler = ipThread;

	// Create Watch Thread
	AfxBeginThread( CThreadManager::WatchThread, ( LPVOID )lWatchThreadParam, ARRESTEXEAPP_THREAD_PRIORITY );
}

UINT CThreadManager::WatchThread( LPVOID pParam )
{
	// Get Thread Info
	SWatchThreadParam* lWatchThreadParam = ( SWatchThreadParam* )pParam;
	CThreadManager* lThreadManager = lWatchThreadParam->m_ThreadManager;
	CThreadHandler* lThreadHandler = lWatchThreadParam->m_ThreadHandler;
	CPtrArray* lThreadArray = &( lThreadManager->m_ThreadArray );
	delete lWatchThreadParam;

	// Wait For Thread Finish
	WaitForSingleObject( lThreadHandler->GetHandle(), INFINITE );

	// Remove Thread From Array
	lThreadManager->m_CriticalSection.Lock();
	int lThreadCount = ( int )lThreadArray->GetCount();
	for( int i = 0; i < lThreadCount; i++ )
	{
		CThreadHandler* lThreadInArray = ( CThreadHandler* )lThreadArray->GetAt( i );
		if( lThreadInArray == lThreadHandler )
		{
			lThreadArray->RemoveAt( i );
			delete lThreadHandler;	// Thread's Destructor Lock This Thread. Be Care!
			break;
		}
	}
	lThreadManager->m_CriticalSection.Unlock();

	return 0;
}

int CThreadManager::GetActiveThreadCount()
{
	return ( int ) m_ThreadArray.GetCount();
}

UINT CThreadManager::RunThread( LPVOID pParam )
{
	return ( ( CThreadHandler* )pParam )->Run();
}

